
/**
 * Module dependencies.
 */
var daemon = require('daemon')
  , express = require('express')
  , routes = require('./routes')
  , util  = require('util')
  , redis = require("redis")
  , io = require('socket.io')
  , fs = require('fs')
  , exec = require('child_process').exec
  ,_ = require('underscore');

var createWebServer = function(){

	var app = module.exports = express.createServer();
	app.register('.html', {
		compile: function(str, options){
			try {
				var template = _.template(str);
				return function(locals){
					return template(locals);
				};
			} 
			catch (e) {
				console.log(e);
			}
		}
	});
	
	
	// Configuration
	app.set('view options', {
		layout: false
	});
	app.configure(function(){
		app.set('views', __dirname + '/views');
		//app.set('view engine', 'jade');
		app.set('view engine', 'html');
		app.use(express.bodyParser());
		app.use(express.methodOverride());
		app.use(express.logger());
		app.use(express.bodyParser());
		app.use(express.cookieParser());
		//app.use(express.session());
		app.use(app.router);
		app.use(express.static(__dirname + '/public'));
		app.use(express.errorHandler());
		
	});
	
	app.configure('development', function(){
		app.use(express.errorHandler({
			dumpExceptions: true,
			showStack: true
		}));
	});
	
	app.configure('production', function(){
		app.use(express.errorHandler());
	});
	
	var showLogs = function(req, res){
	
	
		/*    r.stream.on( 'connect', function() {		
		 console.log("node js db connected");
		 r.sort( 'id:1:log', function( err, data ) {
		 console.log("nodejs sorting");
		 if( !data ) {
		 res.writeHead( 404 );
		 res.write( "No such snippet" );
		 res.end();
		 r.quit();
		 return;
		 }
		
		 console.log("nodejs data", data);
		 res.writeHead( 200 );
		 res.write( 'data' );
		 res.write(util.inspect(data));
		 res.end();
		 r.quit();
		 });
	 });*/
		
		
	}
	
	
	// Routes
	
	app.get('/', function(req, res){
        res.render('dashboard', {});
    });
    app.get('/version', function(req, res){
        //res.set("Content-Type", "text/plain");
        res.header("Content-Type", "text/plain");
        res.sendfile('/opt/teradek/sputnik/VERSION');
    });
	app.get('/session', function(req, res){
		res.render('session', {});
	});
	
	app.get('/session/:sessID', function(req, res){
		res.render('session', {
			sessID: req.params.sessID
		});
	});
	
	app.get('/session/:sessID/:connID', function(req, res){
		res.render('connection', {
			sessID: req.params.sessID
			,connID: req.params.connID
			,startTime: req.params.startTime
			,endTime:   req.params.endTime
			
		});
	});
	
    app.get('/config/viewOnly', function(req,res){
       res.writeHead(200, {"Content-Type": "text/plain"});
        res.write(config.viewOnly.toString());
        res.end();    
    })
    
    app.get('/restart', function(req, res){
        if( config.viewOnly ){
            res.writeHead(200, {"Content-Type": "application/json"});
            res.write('{"error":"You are not authorized to restart the server."}');
            res.end();
            return false;
        }
        exec("service sputnik restart-bond-server", function(error, stdout, stderr){
			    res.writeHead(200, {"Content-Type": "application/json"});
				if (error) {
					res.write('{"error":"' + error + '"}');
				}
				else{
					var output = stdout.replace(/\n/g, " ");
					res.write('{"success":"' + output + '"}');
				}
                res.end();
		});
    });
	
	app.get('/dashboard', function(req, res){
        res.render('dashboard', {});
	});
	
	var dbc = redis.createClient();
    dbc.on("error", function(err){ 
         console.log( 'database error:', err);
         return false;
    })

	
	app.get('/json/sessions', function(req, res){
        res.writeHead(200, {"Content-Type": "application/json"});
		try{
			dbc.keys('sess:*:conn:*:stats',
			         function(err, sessionIDs){
					 	if( err ){ console.log( 'error:', err); return false;}
					 	var response = "[ ";
						sessionIDs.forEach(function(sessionID, connIndex){
							var matches = sessionID.match(/sess:([0-9]+):conn:[^t].*/);
							if( matches && matches[1] ){
                                response += '{"sessionID":"'+ matches[1] + '"},';	
							}
                         });
						response = response.substring(0, response.length - 1);
                        response += "]";
                        res.write(response);
                        res.end();

				});
        }
        catch(e){
            res.write('{"error":"unable to create database client"}');res.end();
            return false;
        }
	});
	
	app.get('/uptime/:sessID', function(req, res){
		res.writeHead(200, {"Content-Type": "application/json"});
        var sessionID = req.params.sessID; 
		dbc.zrange('sess:' + sessionID + ':conn:total:stats', 0, 0, function(err, reply){
			if (reply && reply[0] && typeof(reply[0]) == 'string') {
				res.write(reply[0]);
			}			
			res.end();
		});
		
		
	});
	
    app.get('/time', function(req, res){
        res.writeHead(200, {"Content-Type": "text/plain"});
        var ts = new Date().getTime();
        res.write(ts.toString());
        res.end();
    });
    
	/*
	 *  note: important to distinguish that 'connection' is not the same as 'port(ppp#)'
	 *  as theoretically you may have multiple connections per port
	 *  but as a shortcut connection is ignored
	 */
	app.get('/json/:sessID', function(req, res){
        res.writeHead(200, {"Content-Type": "application/json"});
		var startTime = req.query['startTime'];
		var endTime = req.query['endTime'];
		if( !(endTime) ){
		  endTime = "+inf";
		}
		var sessionID = req.params.sessID; 
		dbc.keys('sess:' + sessionID + ':conn:*', function(err, conns){
			if (!(conns) || !(conns.length)) {
				console.log('no connections found', err);
				if ( err ){
				    res.write('{"error":"'+err+'"}');
				}
				else{
					res.write("{}");
				}
				res.end();
				return false;
			}
			var response = "{";
			connsLength = conns.length;
			if( err ){ console.log( 'error:', err); return false;}
			conns.forEach(function(connKey, connIndex){
				dbc.zrangebyscore(connKey, startTime, endTime, function(err, replies){
					if(connKey.search('total') != -1){
						var portName = 'total';
					}
					else{
						var portName = "Unknown";
					   replies.every(function(replyStr){
					   	 var firstReply = JSON.parse(replyStr);

						 if( firstReply.name && firstReply.name != "" ){
							  portName = firstReply.name;
							  return false;
						 }
						 return true;
					   })
					}
					response += '"' + portName + '":';
					response += '[ ';
					replies.forEach(function(reply, i){
						response += reply + ',';
					});
					response = response.substring(0, response.length - 1);
					response += '],';
					//last connection
					if (connIndex == connsLength - 1) {
						response = response.substring(0, response.length - 1);
						response += '}';
						res.write(response);
						res.end();
					}
				});
			});
		});
    });
	
	/*var websocket = io.listen(app);
	websocket.sockets.on('connection', function(socket){
		var r2 = redis.createClient();
		r2.on("error", function(err){
			console.log("Error " + err);
		});*/
		
//		r2.on("pmessage", function(pattern, channel, message){
			//console.log("pmessage - channel " + channel + ": " + message);
			//get session from channel name
//			var sessID = channel.match(/sess:([0-9]+):.*/)[1];
//			socket.emit('sessions', '{"sessID":'+sessID+', "data":'+message+'}');
//			socket.emit('sess:' + sessID + ':*', message);

//			if (channel.search(/^sess:[^:]*:stats$/) != -1) {
//				message = '{ "sessID":"' + sessID + '", "data":' + message + "}";
//				socket.emit('sess:*:stats', message);
//			}
			
//		});
//		r2.psubscribe("server:*");
//	});
	
	
	app.listen(config.port);
	console.log("Express server listening on port %d in %s mode", config.port, app.settings.env);
	
}

var args = process.argv;
var argsObj = {};
var params = [];

for( var i in args ){
  if( args[i].indexOf( "-" ) == 0 ){
		var arg = args[i].substring( 1 ).split( "=" );
		argsObj[arg[0]] = arg[1];
   }
   else{
   	params.push(args[i]);
   }
}

var config = {
	 port:     argsObj['port']      || 1957
	,lockFile: argsObj['pid']       || '/tmp/sputnik-node.pid'
	,outFile:  argsObj['out']       || '/tmp/sputnik-node-output.log'
	,errFile:  argsObj['err']       || null
	,viewOnly: false
};

if( argsObj.hasOwnProperty('view-only') ){
    config.viewOnly = true;    
}

switch(params[2]) {
	case "stop":
	    daemon.kill( config.lockFile, function( err, pid ){
	      if (err) {
	        return util.puts('Error stopping daemon: ' + err);
	      }
	      
	      util.puts('Successfully stopped daemon with pid: ' + pid);
	    });
		break;
	case "start":
		var fds = config.errFile ? { stdout: config.outFile, stderr: config.errFile } : config.outFile;
		daemon.daemonize( fds, config.lockFile, function(err, pid ){
		    if (err) {
		      return console.log('Error starting daemon: ' + err);
		    }
		    console.log('Sputnik Node.js Daemon started successfully with pid: ' + pid);
		 });
		 createWebServer();
		break;
	case "starti":
		createWebServer();
		break;
	default:
		util.puts('Usage: [start|starti|stop] [ flags ]');
		util.puts(' -port=number');
		util.puts('        Set the network port number to listen on');
		util.puts(' -pid=file');
		util.puts('        Set the location of the pid file');
		util.puts(' -out=file');
		util.puts('        Set the location of the stdout file');
		util.puts(' -err=file');
		util.puts('        Set the location of the separate stderr file');
        util.puts(' -view-only');
        util.puts('        Prevent changing settings from the dashboard.');		

		process.exit(0);
}
